// ReadMap.h: interface for the CReadMap class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_READMAP_H__3064C121_428C_11D4_9677_0050DADC9708__INCLUDED_)
#define AFX_READMAP_H__3064C121_428C_11D4_9677_0050DADC9708__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include <vector>
#include "mapfile.h"
#include "float3.h"
#include "MetalMap.h"
#include "mapfile.h"
#include "sunparser.h"

class CFileHandler;
class CUnit;
class CSolidObject;
class CFileHandler;
class CLoadSaveInterface;

using namespace std;

class CReadMap  
{
public:
	virtual ~CReadMap();

	void AddGroundBlockingObject(CSolidObject *object);
	void AddGroundBlockingObject(CSolidObject *object, unsigned char *blockingMap);
	void RemoveGroundBlockingObject(CSolidObject *object);
	void MoveGroundBlockingObject(CSolidObject *object, float3 oldPos);
	void OpenBlockingYard(CSolidObject *yard, unsigned char *blockingMap);
	void CloseBlockingYard(CSolidObject *yard);
	CSolidObject* GroundBlocked(float3 pos);
	CSolidObject* GroundBlocked(int mapSquare);
	void CleanBlockingMap(CSolidObject* object);	//Debug

	unsigned int bigtex[16],minimapTex,detailTex;

	static CReadMap* Instance();

	float* heightmap;									//hjd
	float* orgheightmap;							//hjd utan skada
	float* centerheightmap;						//hjd i mitten av squares
	float* halfHeightmap;							//hjd i halva upplsningen (fr los etc)
	float* slopemap;
//	float3* normals;
	float3* facenormals;							//normalerna fr trianglarna (2 per square)
	unsigned char* typemap;						//typ av terrng (skog,vg etc
	unsigned char* heightLinePal;			//palette fr hjd karte visning

	struct TerrainType{
		std::string name;
		float hardness;
		float tankSpeed;
		float kbotSpeed;
		float hoverSpeed;
		float shipSpeed;
		int receiveTracks;
	};

	TerrainType terrainTypes[256];

	CSolidObject** groundBlockingObjectMap;				//Pekar-karta till alla blockerande objekt p marken.
	CMetalMap *metalMap;					//Metal-density/height-map

	float tidalStrength;
	
	float3 groundCols[256];

//	unsigned char* yardmapLevels[5];
	unsigned char yardmapPal[256*3];
	unsigned int mapChecksum;
protected:
	int CReadMap::LoadInt(CFileHandler* ifs);
	CReadMap();
	static CReadMap* _instance;
public:
	virtual void RecalcTexture(int x1, int x2, int y1, int y2)=0;
	virtual void Update(){};
	virtual void Explosion(float x,float y,float strength){};
	virtual void ExplosionUpdate(int x1,int x2,int y1,int y2){};
//	void AddYardmap(CSolidObject* unit,bool openYardmap);
//	void RemoveYardmap(CSolidObject* unit,bool openYardmap);

	MapHeader header;
	CFileHandler* ifs;
	CSunParser mapDefParser;

	float3 ambientColor;
	float3 sunColor;
	float shadowDensity;
	float extractorRadius;			//extraction radius for mines
	bool voidWater;

	float minheight,maxheight;
	void LoadSaveMap(CLoadSaveInterface* file,bool loading);

	std::string skyBox;
};

extern CReadMap* readmap;

//Converts a map-square into a float3-position.
inline float3 SquareToFloat3(int xSquare, int zSquare) {return float3(((xSquare))*SQUARE_SIZE, readmap->centerheightmap[(xSquare) + (zSquare) * gs->mapx], ((zSquare))*SQUARE_SIZE);};


#endif // !defined(AFX_READMAP_H__3064C121_428C_11D4_9677_0050DADC9708__INCLUDED_)
